
#ifndef BOOST_MPL_SHIFT_RIGHT_HPP_INCLUDED
#define BOOST_MPL_SHIFT_RIGHT_HPP_INCLUDED

// Copyright Aleksey Gurtovoy 2000-2004
// Copyright Jaap Suter 2003
//
// Distributed under the Boost Software License, Version 1.0. 
// (See accompanying file LICENSE_1_0.txt or copy at 
// http://www.boost.org/LICENSE_1_0.txt)
//
// See http://www.boost.org/libs/mpl for documentation.

// $Id: shift_right.hpp 49267 2008-10-11 06:19:02Z agurtovoy $
// $Date: 2008-10-11 02:19:02 -0400 (Sat, 11 Oct 2008) $
// $Revision: 49267 $

#define AUX778076_OP_NAME shift_right
#define AUX778076_OP_TOKEN >>
#include <boost/mpl/aux_/shift_op.hpp>

#endif // BOOST_MPL_SHIFT_RIGHT_HPP_INCLUDED
